searchNodes=[{"doc":"The protocol specifying how the respective struct might be used within Iteraptor.Experimental. By implementing this protocol one might change the behaviour of nested objects regarding how they should be iterated through.","ref":"Iteraptable.html","title":"Iteraptable","type":"protocol"},{"doc":"Returns a name of the term to be represented in flatmaps","ref":"Iteraptable.html#name/1","title":"Iteraptable.name/1","type":"function"},{"doc":"Converts a term to a collectable","ref":"Iteraptable.html#to_collectable/1","title":"Iteraptable.to_collectable/1","type":"function"},{"doc":"Converts a term to an enumerable","ref":"Iteraptable.html#to_enumerable/1","title":"Iteraptable.to_enumerable/1","type":"function"},{"doc":"Returns a type understood by Iteraptable","ref":"Iteraptable.html#type/1","title":"Iteraptable.type/1","type":"function"},{"doc":"","ref":"Iteraptable.html#t:t/0","title":"Iteraptable.t/0","type":"type"},{"doc":"Iteraptor makes complicated nested structures (currently Maps, Lists and Keywords) iteration easier.UsageIterating, Mapping, ReducingIteraptor.each/3 to iterate a deeply nested map/list/keyword;Iteraptor.map/3 to map a deeply nested map/list/keyword;Iteraptor.reduce/4 to reduce a deeply nested map/list/keyword;Iteraptor.map_reduce/4 to map and reduce a deeply nested map/list/keyword;FlatteningIteraptor.to_flatmap/2 to flatten a deeply nested map/list/keyword into flatten map with concatenated keys;Iteraptor.from_flatmap/3 to “unveil”/“unflatten” the previously flattened map into nested structure;FilteringIteraptor.filter/3 to filter the structure according to the value returned from each iteration (true to leave the element, false to discard.)","ref":"Iteraptor.html","title":"Iteraptor","type":"module"},{"doc":"Iterates the given nested structure, calling the callback provided on each value. The key returned is an array of all the parent keys (and/or indices in a case of an array.)The return value is self.Parametersinput: nested map/list/keyword to be walked through.fun: callback to be called on each {key, value} pair, where key is an array or deeply nested keys; e.g. on %{a: {b: 42}} will be called once, with tuple {[:a, :b], 42};opts: the options to be passed to the iterationyield: [:all | :none | :maps | :lists] what to yield; default: :all for yielding values onlykeys: [:reverse] reverse keys list to ease pattern matching; default: nilstructs: [:values | :keep] how to handle structs; default: nil for treating them as maps. When :values, the nested structs are considered leaves and returned to the iterator instead of being iterated through; when :keep it returns a struct back after iterationExamplesiex&gt; %{a: %{b: %{c: 42}}} |&gt; Iteraptor.each(&amp;IO.inspect/1) {[:a, :b, :c], 42} %{a: %{b: %{c: 42}}} iex&gt; %{a: %{b: %{c: 42}}} |&gt; Iteraptor.each(&amp;IO.inspect/1, yield: :all) {[:a], %{b: %{c: 42}}} {[:a, :b], %{c: 42}} {[:a, :b, :c], 42} %{a: %{b: %{c: 42}}}","ref":"Iteraptor.html#each/3","title":"Iteraptor.each/3","type":"function"},{"doc":"Filters the deeply nested term, optionally calling the function on filtered entries.The return value is the filtered term.Parametersinput: nested map/list/keyword to be filtered.fun: callback to be called on each {key, value} to filter entries.opts: the options to be passed to the iteration (see Iteraptpr.each/3)Examplesiex&gt; %{a: %{b: 42, e: %{f: 3.14, c: 42}, d: %{c: 42}}, c: 42, d: 3.14} ...&gt; |&gt; Iteraptor.filter(fn {key, _} -&gt; :c in key end, yield: :none) %{a: %{e: %{c: 42}, d: %{c: 42}}, c: 42}","ref":"Iteraptor.html#filter/3","title":"Iteraptor.filter/3","type":"function"},{"doc":"Build a nested structure out of a flatmap given, decomposing the names of keys and handling lists carefully.%{&quot;a.b.c&quot;: 42, &quot;a.b.d.0&quot;: nil, &quot;a.b.d.1&quot;: 42, &quot;a.e.0&quot;: :f, &quot;a.e.1&quot;: 42} |&gt; Iteraptor.from_flatmap #⇒ %{a: %{b: %{c: 42, d: [nil, 42]}, e: [:f, 42]}}Parametersinput: flat map to be “expanded” to nested maps/lists,  transformer: the transformer function to be called on all the elements,  opts: additional options to be passed through.Examplesiex&gt; %{&quot;a.b.c&quot;: 42} |&gt; Iteraptor.from_flatmap %{a: %{b: %{c: 42}}} iex&gt; %{&quot;a.b.c&quot;: 42, &quot;a.b.d&quot;: 42} |&gt; Iteraptor.from_flatmap %{a: %{b: %{c: 42, d: 42}}} iex&gt; %{&quot;a.b.c&quot;: 42, &quot;a.b.d&quot;: 42, &quot;a.e&quot;: 42} |&gt; Iteraptor.from_flatmap %{a: %{b: %{c: 42, d: 42}, e: 42}} iex&gt; %{&quot;0&quot;: 42, &quot;1&quot;: 42} |&gt; Iteraptor.from_flatmap [42, 42] iex&gt; %{&quot;1&quot;: :a1, &quot;0&quot;: :a0, &quot;2&quot;: :a2, &quot;3&quot;: :a3, &quot;4&quot;: :a4, &quot;5&quot;: :a5, ...&gt; &quot;6&quot;: :a6, &quot;7&quot;: :a7, &quot;8&quot;: :a8, &quot;9&quot;: :a9, &quot;10&quot;: :a10, &quot;11&quot;: :a11} ...&gt; |&gt; Iteraptor.from_flatmap [:a0, :a1, :a2, :a3, :a4, :a5, :a6, :a7, :a8, :a9, :a10, :a11] iex&gt; %{&quot;0.a&quot;: 42, &quot;0.b&quot;: 42} |&gt; Iteraptor.from_flatmap [%{a: 42, b: 42}] iex&gt; %{&quot;a.0.0&quot; =&gt; :b, &quot;a.1&quot; =&gt; 42, d: 42} |&gt; Iteraptor.from_flatmap %{a: [[:b], 42], d: 42} iex&gt; %{&quot;a.b.c&quot;: 42, &quot;a.b.d.0&quot;: nil, &quot;a.b.d.1&quot;: 42, &quot;a.e.0&quot;: :f, &quot;a.e.1&quot;: 42} ...&gt; |&gt; Iteraptor.from_flatmap %{a: %{b: %{c: 42, d: [nil, 42]}, e: [:f, 42]}} iex&gt; %{&quot;0.a&quot;: 42, &quot;0.b&quot;: 42} |&gt; Iteraptor.from_flatmap(&amp;IO.inspect/1) {[0, :a], 42} {[0, :b], 42} [%{a: 42, b: 42}]","ref":"Iteraptor.html#from_flatmap/3","title":"Iteraptor.from_flatmap/3","type":"function"},{"doc":"Produces a term ready-to-use with JSON interchange. Stringifies all keys and converts keywords to maps.If the option keys: false is given, leaves keys intact.Examplesiex&gt; Iteraptor.jsonify([foo: [:zzz], bar: :baz], values: true) %{&quot;foo&quot; =&gt; [&quot;zzz&quot;], &quot;bar&quot; =&gt; &quot;baz&quot;} iex&gt; Iteraptor.jsonify(%{foo: [1, [bar: 2], 3], bar: [baz: 42]}) %{&quot;foo&quot; =&gt; [1, %{&quot;bar&quot; =&gt; 2}, 3], &quot;bar&quot; =&gt; %{&quot;baz&quot; =&gt; 42}} iex&gt; Iteraptor.jsonify([foo: [bar: [baz: :zoo], boo: 42]], values: true) %{&quot;foo&quot; =&gt; %{&quot;bar&quot; =&gt; %{&quot;baz&quot; =&gt; &quot;zoo&quot;}, &quot;boo&quot; =&gt; 42}} iex&gt; Iteraptor.jsonify([foo: [bar: [baz: :zoo], boo: 42]], keys: false) %{foo: %{bar: %{baz: :zoo}, boo: 42}}","ref":"Iteraptor.html#jsonify/2","title":"Iteraptor.jsonify/2","type":"function"},{"doc":"Maps the given nested structure, calling the callback provided on each value. The key returned is a concatenated names of all the parent keys (and/or indices in a case of an array.)The return value is the result of subsequent calls to the transformer given.Parametersinput: nested map/list/keyword to be mapped.fun: callback to be called on each {key, value} pair, where key is an array or deeply nested keys; e.g. on %{a: {b: 42}} will be called once, with tuple {[:a, :b], 42};opts: the options to be passed to the iteration (see Iteraptpr.each/3)Examplesiex&gt; %{a: %{b: %{c: 42}}} |&gt; Iteraptor.map(fn {_, v} -&gt; v * 2 end) %{a: %{b: %{c: 84}}} iex&gt; %{a: %{b: %{c: 42}}} |&gt; Iteraptor.map(fn {k, _} -&gt; Enum.join(k) end) %{a: %{b: %{c: &quot;abc&quot;}}} iex&gt; %{a: %{b: %{c: 42}}} ...&gt; |&gt; Iteraptor.map(fn ...&gt; {[_], _} = self -&gt; self ...&gt; {[_, _], _} -&gt; &quot;YAY&quot; ...&gt; end, yield: :all) %{a: %{b: &quot;YAY&quot;}}","ref":"Iteraptor.html#map/3","title":"Iteraptor.map/3","type":"function"},{"doc":"Iteration with mapping and reducing. The function of arity 2, called back on each iteration with {k, v} pair and an accumulator is accepted.The return value is the tuple, consisting of mapped input and the accumulator from the last call to the passed map-reducer.Parametersinput: nested map/list/keyword to be mapped.fun: callback to be called on each {key, value}, acc pair, where key is an array or deeply nested keys, value is the value and acc is the accumulator;opts: the options to be passed to the iteration (see Iteraptpr.each/3)Examplesiex&gt; %{a: %{b: %{c: 42}}} ...&gt; |&gt; Iteraptor.map_reduce([], fn ...&gt; {k, %{} = v}, acc -&gt; {{k, v}, [Enum.join(k, &quot;.&quot;) | acc]} ...&gt; {k, v}, acc -&gt; {{k, v * 2}, [Enum.join(k, &quot;.&quot;) &lt;&gt; &quot;=&quot; | acc]} ...&gt; end, yield: :all) {%{a: %{b: %{c: 84}}}, [&quot;a.b.c=&quot;, &quot;a.b&quot;, &quot;a&quot;]}","ref":"Iteraptor.html#map_reduce/4","title":"Iteraptor.map_reduce/4","type":"function"},{"doc":"Iteration with reducing. The function of arity 2, called back on each iteration with {k, v} pair and an accumulator is accepted.The return value is the result of the last call to the passed reducer function.Parametersinput: nested map/list/keyword to be mapped.fun: callback to be called on each {key, value}, acc pair, where key is an array or deeply nested keys, value is the value and acc is the accumulator;opts: the options to be passed to the iteration (see Iteraptpr.each/3)Examplesiex&gt; %{a: %{b: %{c: 42}}} ...&gt; |&gt; Iteraptor.reduce([], fn {k, _}, acc -&gt; ...&gt; [Enum.join(k, &quot;_&quot;) | acc] ...&gt; end, yield: :all) ...&gt; |&gt; :lists.reverse() [&quot;a&quot;, &quot;a_b&quot;, &quot;a_b_c&quot;]","ref":"Iteraptor.html#reduce/4","title":"Iteraptor.reduce/4","type":"function"},{"doc":"Build a flatmap out of nested structure, concatenating the names of keys.%{a: %{b: %{c: 42, d: [nil, 42]}, e: [:f, 42]}} |&gt; Iteraptor.to_flatmap %{&quot;a.b.c&quot;: 42, &quot;a.b.d.0&quot;: nil, &quot;a.b.d.1&quot;: 42, &quot;a.e.0&quot;: :f, &quot;a.e.1&quot;: 42}Lists are handled gracefully, index is used as a key in resulting map.Parametersinput: nested map/list/keyword/struct to be flattened,  opts: the additional options to be passed through:     — delimiter (default: &quot;.&quot;,) might be passed explicitly or configured with :iteraptor, :delimiter application setting.Examplesiex&gt; [:a, 42] |&gt; Iteraptor.to_flatmap %{0 =&gt; :a, 1 =&gt; 42} iex&gt; %{a: 42} |&gt; Iteraptor.to_flatmap %{a: 42} iex&gt; %{a: 42, b: 42} |&gt; Iteraptor.to_flatmap %{a: 42, b: 42} iex&gt; %{a: %{b: 42}, d: 42} |&gt; Iteraptor.to_flatmap %{&quot;a.b&quot; =&gt; 42, d: 42} iex&gt; %{a: [:b, 42], d: 42} |&gt; Iteraptor.to_flatmap %{&quot;a.0&quot; =&gt; :b, &quot;a.1&quot; =&gt; 42, d: 42} iex&gt; %{a: %{b: [:c, 42]}, d: 42} |&gt; Iteraptor.to_flatmap %{&quot;a.b.0&quot; =&gt; :c, &quot;a.b.1&quot; =&gt; 42, d: 42} iex&gt; %{a: %{b: 42}} |&gt; Iteraptor.to_flatmap %{&quot;a.b&quot; =&gt; 42} iex&gt; %{a: %{b: %{c: 42}}} |&gt; Iteraptor.to_flatmap %{&quot;a.b.c&quot; =&gt; 42} iex&gt; %{a: %{b: %{c: 42}}, d: 42} |&gt; Iteraptor.to_flatmap %{&quot;a.b.c&quot; =&gt; 42, d: 42} iex&gt; [a: [b: [c: 42]], d: 42] |&gt; Iteraptor.to_flatmap %{&quot;a.b.c&quot; =&gt; 42, d: 42} iex&gt; [a: [[:b], 42], d: 42] |&gt; Iteraptor.to_flatmap %{&quot;a.0.0&quot; =&gt; :b, &quot;a.1&quot; =&gt; 42, d: 42} iex&gt; %{a: %{b: %{c: 42, d: [nil, 42]}, e: [:f, 42]}} |&gt; Iteraptor.to_flatmap %{&quot;a.b.c&quot; =&gt; 42, &quot;a.b.d.0&quot; =&gt; nil, &quot;a.b.d.1&quot; =&gt; 42, &quot;a.e.0&quot; =&gt; :f, &quot;a.e.1&quot; =&gt; 42} iex&gt; %{a: %{b: %{c: 42, d: [nil, 42]}, e: [:f, 42]}} ...&gt; |&gt; Iteraptor.to_flatmap(delimiter: &quot;_&quot;) %{&quot;a_b_c&quot; =&gt; 42, &quot;a_b_d_0&quot; =&gt; nil, &quot;a_b_d_1&quot; =&gt; 42, &quot;a_e_0&quot; =&gt; :f, &quot;a_e_1&quot; =&gt; 42}","ref":"Iteraptor.html#to_flatmap/2","title":"Iteraptor.to_flatmap/2","type":"function"},{"doc":"","ref":"Iteraptor.html#t:option/0","title":"Iteraptor.option/0","type":"type"},{"doc":"","ref":"Iteraptor.html#t:options/0","title":"Iteraptor.options/0","type":"type"},{"doc":"The function that might be passed to all the traversion functions.When it’s a function or arity 1, it receives {key, value} tuple when the key is the list of keys down the nesting levels.When its arity is 2, it receives key and value as separated arguments.","ref":"Iteraptor.html#t:traverse_fun/0","title":"Iteraptor.traverse_fun/0","type":"type"},{"doc":"Array emulation implementing Access behaviour. Index in array is zero-based.Array is the &quot;go to&quot; array data structure in Elixir. An array can be constructed using Array.new/{0,1}:iex&gt; Iteraptor.Array.new() #Array&lt;[]&gt; iex&gt; Iteraptor.Array.new(2) #Array&lt;[nil, nil]&gt; iex&gt; Iteraptor.Array.new([:foo, :bar]) #Array&lt;[:foo, :bar]&gt;An array can contain any kind of elements, and elements in an array don't have to be of the same type. By definition, arrays have keys in 0..size-1 range. Arrays are implicitly expandable, which means adding an element at index 100 to the array currently containing 1 element would increase the size of the array to 100.iex&gt; array = Iteraptor.Array.new([:foo]) iex&gt; Iteraptor.Array.set(array, 3, :bar) #Array&lt;[:foo, nil, nil, :bar]&gt;An Array is represented internally using the %Array{} struct. Note that, however, the struct fields are private and must not be accessed directly; use the functions in this module to perform operations on arrays.Arrays can also be constructed starting from other collection-type data structures: for example, see Array.new/1 or Enum.into/2.iex&gt; Enum.into([1, 2, 3], Iteraptor.Array.new()) #Array&lt;[1, 2, 3]&gt;Arrays do implement Access behaviour.iex&gt; array = Iteraptor.Array.new([%{foo: 42}, %{bar: :baz}]) iex&gt; get_in(array, [0, :foo]) 42","ref":"Iteraptor.Array.html","title":"Iteraptor.Array","type":"module"},{"doc":"Appends another enumerable to the array.iex&gt; array = Iteraptor.Array.new([1, 2, 3]) iex&gt; Iteraptor.Array.append(array, [4, 5]) #Array&lt;[1, 2, 3, 4, 5]&gt;","ref":"Iteraptor.Array.html#append/2","title":"Iteraptor.Array.append/2","type":"function"},{"doc":"Converts a tuple given as parameter to array.iex&gt; Iteraptor.Array.from_tuple({1, 2, 3}) #Array&lt;[1, 2, 3]&gt;","ref":"Iteraptor.Array.html#from_tuple/1","title":"Iteraptor.Array.from_tuple/1","type":"function"},{"doc":"Returns the value at index in array, or default if index is out of array bounds.iex&gt; array = Iteraptor.Array.new([42]) iex&gt; Iteraptor.Array.get(array, 0) 42 iex&gt; Iteraptor.Array.get(array, 2, 42) 42","ref":"Iteraptor.Array.html#get/3","title":"Iteraptor.Array.get/3","type":"function"},{"doc":"Returns a new array.iex&gt; Iteraptor.Array.new() #Array&lt;[]&gt;Creates an array of the given length or from enumerable. Might we used to wrap the existing instance of Iteraptor.Array.iex&gt; Iteraptor.Array.new(3) #Array&lt;[nil, nil, nil]&gt; iex&gt; Iteraptor.Array.new([:foo, :bar, 42]) #Array&lt;[:foo, :bar, 42]&gt;Also the transformation function might be passed via second argument.iex&gt; Iteraptor.Array.new([1, 2, 3], fn x -&gt; 2 * x end) #Array&lt;[2, 4, 6]&gt;","ref":"Iteraptor.Array.html#new/2","title":"Iteraptor.Array.new/2","type":"function"},{"doc":"Pops (deletes) value at index from array, setting the value at the respective index to nil. Returns a tuple containing the value removed and the new array.iex&gt; array = Iteraptor.Array.new([1, 2, 3]) iex&gt; {elem, array} = Iteraptor.Array.pop(array, 1) iex&gt; elem 2 iex&gt; array #Array&lt;[1, nil, 3]&gt;","ref":"Iteraptor.Array.html#pop/2","title":"Iteraptor.Array.pop/2","type":"function"},{"doc":"Sets the value at index in array, expanding the array if necessary. Returns a new array.iex&gt; array = Iteraptor.Array.new([42]) iex&gt; Iteraptor.Array.set(array, 0, :foo) #Array&lt;[:foo]&gt; iex&gt; Iteraptor.Array.set(array, 2, :bar) #Array&lt;[42, nil, :bar]&gt;","ref":"Iteraptor.Array.html#set/3","title":"Iteraptor.Array.set/3","type":"function"},{"doc":"Returns the number of elements in array.iex&gt; Iteraptor.Array.size(Iteraptor.Array.new([1, 2, 3])) 3","ref":"Iteraptor.Array.html#size/1","title":"Iteraptor.Array.size/1","type":"function"},{"doc":"Converts array to a list.iex&gt; Iteraptor.Array.to_list(Iteraptor.Array.new([1, 2, 3])) [1, 2, 3]","ref":"Iteraptor.Array.html#to_list/1","title":"Iteraptor.Array.to_list/1","type":"function"},{"doc":"Trims nil values from the tail of the Array. Returns a trimmed array.iex&gt; array = Iteraptor.Array.new([42, nil, nil]) #Array&lt;[42, nil, nil]&gt; iex&gt; Iteraptor.Array.trim(array) #Array&lt;[42]&gt;","ref":"Iteraptor.Array.html#trim/1","title":"Iteraptor.Array.trim/1","type":"function"},{"doc":"","ref":"Iteraptor.Array.html#t:t/0","title":"Iteraptor.Array.t/0","type":"type"},{"doc":"","ref":"Iteraptor.Array.html#t:value/0","title":"Iteraptor.Array.value/0","type":"type"},{"doc":"Extra functions to deal with runtime configuration.","ref":"Iteraptor.Config.html","title":"Iteraptor.Config","type":"module"},{"doc":"Reads the application config and recursively changes all the {:key, &quot;VAR&quot;} tuples with runtime system environment read from &quot;VAR&quot;.Default value for :key is :system, but it might be adjusted to avoid clashes with other libraries implementing {:system, &quot;FOO&quot;} functionality, like Phoenix and/or Ecto.Examples:iex&gt; System.put_env(&quot;FOO&quot;, &quot;42&quot;) iex&gt; Application.put_env( ...&gt; :iteraptor, ...&gt; :key, ...&gt; [value: [value: {:my_system, &quot;FOO&quot;}]], ...&gt; persistent: true ...&gt; ) iex&gt; Application.get_all_env(:iteraptor) [key: [value: [value: {:my_system, &quot;FOO&quot;}]]] iex&gt; Iteraptor.Config.from_env(:iteraptor, &amp;String.to_integer/1, :my_system) iex&gt; Application.get_all_env(:iteraptor) [key: [value: [value: 42]]]","ref":"Iteraptor.Config.html#from_env/3","title":"Iteraptor.Config.from_env/3","type":"function"},{"doc":"Extra functions to deal with enumerables.","ref":"Iteraptor.Extras.html","title":"Iteraptor.Extras","type":"module"},{"doc":"Deep store the value into the nested structure. Behaves as a proposed but rejected in ruby core Hash#bury.Examples:iex&gt; Iteraptor.Extras.bury(%{foo: :bar}, ~w|a b c d|a, 42) %{a: %{b: %{c: %{d: 42}}}, foo: :bar} iex&gt; Iteraptor.Extras.bury([foo: :bar], ~w|a b c d|a, 42) [a: [b: [c: [d: 42]]], foo: :bar] iex&gt; Iteraptor.Extras.bury(%{foo: :bar}, ~w|a b c d|a, 42, into: :keyword) %{a: [b: [c: [d: 42]]], foo: :bar} iex&gt; Iteraptor.Extras.bury(42, ~w|a b c d|a, 42) ** (Iteraptor.Utils.Unsupported) Unsupported term 42 in call to Iteraptor.Extras.bury/4.","ref":"Iteraptor.Extras.html#bury/4","title":"Iteraptor.Extras.bury/4","type":"function"},{"doc":"Behaves as Enum.each_cons(n) in ruby. Iterates the input producing the list of cons. Gracefully stolen from https://groups.google.com/forum/#!topic/elixir-lang-core/LAK23vaJgvEExamplesiex&gt; &#39;letters&#39; |&gt; Iteraptor.Extras.each_cons [&#39;le&#39;, &#39;et&#39;, &#39;tt&#39;, &#39;te&#39;, &#39;er&#39;, &#39;rs&#39;] iex&gt; &#39;letters&#39; |&gt; Iteraptor.Extras.each_cons(4) [&#39;lett&#39;, &#39;ette&#39;, &#39;tter&#39;, &#39;ters&#39;] iex&gt; 1..6 |&gt; Iteraptor.Extras.each_cons(4) [[1,2,3,4], [2,3,4,5], [3,4,5,6]] iex&gt; &quot;letters&quot; |&gt; Iteraptor.Extras.each_cons(3) [&quot;let&quot;, &quot;ett&quot;, &quot;tte&quot;, &quot;ter&quot;, &quot;ers&quot;]","ref":"Iteraptor.Extras.html#each_cons/3","title":"Iteraptor.Extras.each_cons/3","type":"function"},{"doc":"use Iteraptor.Iteraptable inside structs to make them both Enumerable and Collectable and implement the Access behaviour:UsageUse the module within the struct of your choice and this struct will be automagically granted Enumerable and Collectable protocols implementations.use Iteraptor.Iteraptable accepts keyword parameter skip: Access or skip: [Enumerable, Collectable] which allows to implement a subset of protocols. Also it accepts keyword parameter derive: MyProtocol allowing to specify what protocol(s) implementations should be implicitly derived for this struct.","ref":"Iteraptor.Iteraptable.html","title":"Iteraptor.Iteraptable","type":"module"},{"doc":"Allows to enable iterating features on structs with use Iteraptor.IteraptableParameterskeyword parameter optsskip: Access or skip: [Enumerable, Collectable] allows to implement a subset of protocols;derive: MyProtocol allows to derive selected protocol implementation(s).","ref":"Iteraptor.Iteraptable.html#__using__/1","title":"Iteraptor.Iteraptable.__using__/1","type":"macro"},{"doc":"Iteraptor.AST module traverses AST, allowing map, reduce and family.","ref":"Iteraptor.AST.html","title":"Iteraptor.AST","type":"module"},{"doc":"Mapper for the AST.Parametersinput: the AST to traversefun: the function to be called on the tree elementopts: the options to be passed to the iterationyield: :all | nil what to yield; default: nil for yielding values only.Examplesiex&gt; bindings = [a: 1, b: 2, c: 3] ...&gt; &quot;:math.sin(42 * a / (3.14 * b)) &gt; c&quot; ...&gt; |&gt; Iteraptor.AST.map(fn ...&gt; {var, _, val} when is_atom(val) -&gt; bindings[var] ...&gt; any -&gt; any ...&gt; end) {:&gt;, [line: 1], [ {{:., [line: 1], [:math, :sin]}, [line: 1], [ {:/, [line: 1], [ {:*, [line: 1], [42, 1]}, {:*, [line: 1], [3.14, 2]} ]} ]}, 3 ]}","ref":"Iteraptor.AST.html#map/3","title":"Iteraptor.AST.map/3","type":"function"},{"doc":"Reduces the AST with an accumulator.Parametersinput: the AST to traverseacc: the accumulatorfun: the function to be called on the tree elementopts: the options to be passed to the iterationyield: :all | nil what to yield; default: nil for yielding values only.Examplesiex&gt; &quot;:math.sin(42 * a / (3.14 * b)) &gt; c&quot; ...&gt; |&gt; Iteraptor.AST.reduce([], fn ...&gt; {var, _, val}, acc when is_atom(val) -&gt; [var | acc] ...&gt; _, acc -&gt; acc ...&gt; end) ...&gt; |&gt; Enum.reverse() ~w|a b c|a","ref":"Iteraptor.AST.html#reduce/4","title":"Iteraptor.AST.reduce/4","type":"function"},{"doc":"Helper functions to update nested terms","ref":"Iteraptor.Utils.html","title":"Iteraptor.Utils","type":"module"},{"doc":"Safe put the value deeply into the term nesting structure. Creates all the intermediate keys if needed.Examples:iex&gt; Iteraptor.Utils.deep_put_in(%{}, {~w|a b c|a, 42}) %{a: %{b: %{c: 42}}} iex&gt; Iteraptor.Utils.deep_put_in(%{a: %{b: %{c: 42}}}, {~w|a b d|a, :foo}) %{a: %{b: %{c: 42, d: :foo}}} iex&gt; Iteraptor.Utils.deep_put_in(%{a: %{b: [c: 42]}}, {~w|a b d|a, :foo}) %{a: %{b: [c: 42, d: :foo]}} iex&gt; Iteraptor.Utils.deep_put_in(%{a: %{b: [42]}}, {~w|a b|a, :foo}) %{a: %{b: [42, :foo]}} iex&gt; Iteraptor.Utils.deep_put_in(%{a: [:foo, %{b: 42}]}, {~w|a b|a, :foo}) %{a: [:foo, %{b: 42}, {:b, :foo}]}","ref":"Iteraptor.Utils.html#deep_put_in/3","title":"Iteraptor.Utils.deep_put_in/3","type":"function"},{"doc":"Digs the leaf value in the nested keyword / map.Examples:iex&gt; Iteraptor.Utils.dig(%{k1: %{k2: %{k3: :value}}}) {:ok, {[:k1, :k2, :k3], :value}} iex&gt; Iteraptor.Utils.dig([k1: [k2: [k3: :value]]]) {:ok, {[:k1, :k2, :k3], :value}} iex&gt; Iteraptor.Utils.dig([k1: :value, k2: :value]) {:error, [k1: :value, k2: :value]} iex&gt; Iteraptor.Utils.dig([k1: %{k2: [k3: :value]}]) {:ok, {[:k1, :k2, :k3], :value}}","ref":"Iteraptor.Utils.html#dig/2","title":"Iteraptor.Utils.dig/2","type":"function"},{"doc":"","ref":"Iteraptor.Utils.html#dig!/2","title":"Iteraptor.Utils.dig!/2","type":"function"},{"doc":"Joins the array of keys into the string using delimiter.Examples:iex&gt; Iteraptor.Utils.join(~w|a b c d|) &quot;a.b.c.d&quot; iex&gt; Iteraptor.Utils.join(~w|a b c d|, delimiter: &quot;_&quot;) &quot;a_b_c_d&quot;","ref":"Iteraptor.Utils.html#join/2","title":"Iteraptor.Utils.join/2","type":"function"},{"doc":"Checks if the map/keyword looks like a normal list.Examples:iex&gt; Iteraptor.Utils.quacks_as_list(%{&quot;0&quot; =&gt; :foo, 1 =&gt; :bar}) true iex&gt; Iteraptor.Utils.quacks_as_list([{:&quot;1&quot;, :bar}, {:&quot;0&quot;, :foo}]) true iex&gt; Iteraptor.Utils.quacks_as_list(%{foo: :bar}) false iex&gt; Iteraptor.Utils.quacks_as_list(%{&quot;5&quot; =&gt; :foo, &quot;1&quot; =&gt; :bar}) false iex&gt; Iteraptor.Utils.quacks_as_list(42) false","ref":"Iteraptor.Utils.html#quacks_as_list/1","title":"Iteraptor.Utils.quacks_as_list/1","type":"function"},{"doc":"Splits the string by delimiter, possibly converting the keys to symbols.Examples:iex&gt; Iteraptor.Utils.split(&quot;a.b.c.d&quot;, transform: :none) [&quot;a&quot;, &quot;b&quot;, &quot;c&quot;, &quot;d&quot;] iex&gt; Iteraptor.Utils.split(&quot;a_b_c_d&quot;, delimiter: &quot;_&quot;) [&quot;a&quot;, &quot;b&quot;, &quot;c&quot;, &quot;d&quot;] iex&gt; Iteraptor.Utils.split(&quot;a.b.c.d&quot;, transform: :unsafe) [:a, :b, :c, :d] iex&gt; Iteraptor.Utils.split(&quot;a.b.c.d&quot;, transform: :safe) [:a, :b, :c, :d]","ref":"Iteraptor.Utils.html#split/2","title":"Iteraptor.Utils.split/2","type":"function"},{"doc":"Squeezes the nested structure merging same keys.Examples:#iex&gt; Iteraptor.Utils.squeeze([foo: [bar: 42], foo: [baz: 3.14]]) #[foo: [bar: 42, baz: 3.14]] iex&gt; Iteraptor.Utils.squeeze([foo: %{bar: 42}, foo: %{baz: 3.14}]) [foo: %{bar: 42, baz: 3.14}] iex&gt; Iteraptor.Utils.squeeze([foo: %{bar: 42}, foo: :baz]) [foo: [%{bar: 42}, :baz]] iex&gt; Iteraptor.Utils.squeeze([a: [b: [c: 42]], a: [b: [d: 3.14]]]) [a: [b: [c: 42, d: 3.14]]] iex&gt; Iteraptor.Utils.squeeze([a: [b: [c: 42]], a: [b: %{d: 3.14}]]) [a: [b: [c: 42, d: 3.14]]] iex&gt; Iteraptor.Utils.squeeze([a: [b: [c: :foo]], a: [b: [c: 3.14]]]) [a: [b: [c: [:foo, 3.14]]]] iex&gt; Iteraptor.Utils.squeeze([a: [b: [:foo, :bar]], a: [b: [c: 3.14]]]) [a: [b: [:foo, :bar, {:c, 3.14}]]] iex&gt; Iteraptor.Utils.squeeze([a: [:foo, :bar], a: [b: [c: 3.14]]]) [a: [:foo, :bar, {:b, [c: 3.14]}]]","ref":"Iteraptor.Utils.html#squeeze/2","title":"Iteraptor.Utils.squeeze/2","type":"function"},{"doc":"Gently tries to create a linked list out of input, returns input if it cannot be safely converted to the list.Examples:iex&gt; Iteraptor.Utils.try_to_list(%{&quot;0&quot; =&gt; :foo, 1 =&gt; :bar}) [:foo, :bar] iex&gt; Iteraptor.Utils.try_to_list([{:&quot;1&quot;, :bar}, {:&quot;0&quot;, :foo}]) [:foo, :bar] iex&gt; Iteraptor.Utils.try_to_list(%{foo: :bar}) %{foo: :bar} iex&gt; Iteraptor.Utils.try_to_list(%{&quot;5&quot; =&gt; :foo, &quot;1&quot; =&gt; :bar}) %{&quot;5&quot; =&gt; :foo, &quot;1&quot; =&gt; :bar}","ref":"Iteraptor.Utils.html#try_to_list/1","title":"Iteraptor.Utils.try_to_list/1","type":"function"},{"doc":"Determines the type of the given term.Examples:iex&gt; Iteraptor.Utils.type(%{foo: :bar}) {Map, %{foo: :bar}, %{}} iex&gt; Iteraptor.Utils.type([foo: :bar]) {Keyword, [foo: :bar], []} iex&gt; Iteraptor.Utils.type([{:foo, :bar}]) {Keyword, [{:foo, :bar}], []} iex&gt; Iteraptor.Utils.type(~w|foo bar|a) {List, [:foo, :bar], []} iex&gt; Iteraptor.Utils.type(42) :error","ref":"Iteraptor.Utils.html#type/1","title":"Iteraptor.Utils.type/1","type":"function"},{"doc":"Unsupported in applying Iteraptor.Iteraptable","ref":"Iteraptor.Iteraptable.Unsupported.html","title":"Iteraptor.Iteraptable.Unsupported","type":"exception"},{"doc":"An exception to be thrown from banged methods of Iteraptor.Sooner or later we’ll support everything, that’s why meanwhile we raise Unsupported if something goes wrong.","ref":"Iteraptor.Utils.Unsupported.html","title":"Iteraptor.Utils.Unsupported","type":"exception"},{"doc":"Iteraptor","ref":"intro.html","title":"Iteraptor","type":"extras"},{"doc":"The tiny elixir library to iterate/map/reduce/filter deeply nested structures in Elixir.This package is a sibling of Ruby Iteraptor gem.","ref":"intro.html#iterating-nested-terms-like-i-m-five","title":"Iteraptor - Iterating Nested Terms Like I’m Five","type":"extras"},{"doc":"source code;documentation.","ref":"intro.html#tl-dr","title":"Iteraptor - TL;DR","type":"extras"},{"doc":"Iterating both maps and lists in Elixir is charming. One might chain iterators, map, reduce, filter, select, reject, zip... Everybody having at least eight hours of experience with Elixir has definitely seen (and even maybe written) something like this:~w|aleksei saverio| |&gt; Enum.map(&amp; String.capitalize/1) |&gt; Enum.each(fn capitalized_name -&gt; IO.puts &quot;Hello, \#{capitalized_name}!&quot; end)That is really handy. The things gets cumbersome when it comes to deeply nested structures, like a map having nested keywords, lists etc. The good example of that would be any configuration file, having nested subsections.While Elixir provides helpers to update elements deeply inside such a term:Kernel.get_in/2Kernel.put_in/{2,3}Kernel.update_in/{2,3}Kernel.get_and_update_in/{2,3}all the above would work if and only all the parent levels in the structure exist. The exception would be get_in/2 which is happily returning nil being asked for whatever inexisting.The amount of questions on Stack Overflow asking “how would I modify a nested structure” forced me to finally create this library. The implementation in Elixir looks a bit more convoluted since everything is immutable and one cannot just traverse a structure down to leaves, modifying whatever needed in-place. The iteration-wide accumulator is required.That is probably the only example I met in my life where mutability makes things easier. As a bonus the implementation of bury/4 to store the value deeply inside a structure, creating the intermediate keys as necessary, was introduced. It behaves as a proposed but rejected in ruby core Hash#bury.So, welcome the library that makes the iteration of any nested map/keyword/list combination almost as easy as the natural Elixir map and each.• Iteraptor","ref":"intro.html#intro","title":"Iteraptor - Intro","type":"extras"},{"doc":"Iteraptor.each/3 to iterate a deeply nested map/list/keyword;Iteraptor.map/3 to map a deeply nested map/list/keyword;Iteraptor.reduce/4 to reduce a deeply nested map/list/keyword;Iteraptor.map_reduce/4 to map and reduce a deeply nested map/list/keyword;Iteraptor.filter/3 to filter a deeply nested map/list/keyword;Iteraptor.to_flatmap/2 to flatten a deeply nested map/list/keyword into flatten map with concatenated keys;Iteraptor.from_flatmap/3 to “unveil”/“unflatten” the previously flattened map into nested structure;use Iteraptor.Iteraptable to automagically implement Enumerable and Collectable protocols, as well as Access behaviour on the structure.","ref":"intro.html#features","title":"Iteraptor - Features","type":"extras"},{"doc":"Iterating, Mapping, Reducing# each iex&gt; %{a: %{b: %{c: 42}}} |&gt; Iteraptor.each(&amp;IO.inspect/1, yield: :all) # {[:a], %{b: %{c: 42}}} # {[:a, :b], %{c: 42}} # {[:a, :b, :c], 42} %{a: %{b: %{c: 42}}} # map iex&gt; %{a: %{b: %{c: 42}}} |&gt; Iteraptor.map(fn {k, _} -&gt; Enum.join(k) end) %{a: %{b: %{c: &quot;abc&quot;}}} iex&gt; %{a: %{b: %{c: 42}}} ...&gt; |&gt; Iteraptor.map(fn ...&gt; {[_], _} = self -&gt; self ...&gt; {[_, _], _} -&gt; &quot;YAY&quot; ...&gt; end, yield: :all) %{a: %{b: &quot;YAY&quot;}} # reduce iex&gt; %{a: %{b: %{c: 42}}} ...&gt; |&gt; Iteraptor.reduce([], fn {k, _}, acc -&gt; ...&gt; [Enum.join(k, &quot;_&quot;) | acc] ...&gt; end, yield: :all) ...&gt; |&gt; :lists.reverse() [&quot;a&quot;, &quot;a_b&quot;, &quot;a_b_c&quot;] # map-reduce iex&gt; %{a: %{b: %{c: 42}}} ...&gt; |&gt; Iteraptor.map_reduce([], fn ...&gt; {k, %{} = v}, acc -&gt; {​{k, v}, [Enum.join(k, &quot;.&quot;) | acc]} ...&gt; {k, v}, acc -&gt; {​{k, v * 2}, [Enum.join(k, &quot;.&quot;) &lt;&gt; &quot;=&quot; | acc]} ...&gt; end, yield: :all) {​%{a: %{b: %{c: 42}}}, [&quot;a.b.c=&quot;, &quot;a.b&quot;, &quot;a&quot;]} # filter iex&gt; %{a: %{b: 42, e: %{f: 3.14, c: 42}, d: %{c: 42}}, c: 42, d: 3.14} ...&gt; |&gt; Iteraptor.filter(fn {key, _} -&gt; :c in key end, yield: :none) %{a: %{e: %{c: 42}, d: %{c: 42}}, c: 42}Flatteningiex&gt; %{a: %{b: %{c: 42, d: [nil, 42]}, e: [:f, 42]}} ...&gt; |&gt; Iteraptor.to_flatmap(delimiter: &quot;_&quot;) #⇒ %{&quot;a_b_c&quot; =&gt; 42, &quot;a_b_d_0&quot; =&gt; nil, &quot;a_b_d_1&quot; =&gt; 42, &quot;a_e_0&quot; =&gt; :f, &quot;a_e_1&quot; =&gt; 42} iex&gt; %{&quot;a.b.c&quot;: 42, &quot;a.b.d.0&quot;: nil, &quot;a.b.d.1&quot;: 42, &quot;a.e.0&quot;: :f, &quot;a.e.1&quot;: 42} ...&gt; |&gt; Iteraptor.from_flatmap #⇒ %{a: %{b: %{c: 42, d: [nil, 42]}, e: [:f, 42]}}Extrasiex&gt; Iteraptor.Extras.bury([foo: :bar], ~w|a b c d|a, 42) [a: [b: [c: [d: 42]]], foo: :bar]","ref":"intro.html#words-are-cheap-show-me-the-code","title":"Iteraptor - Words are cheap, show me the code","type":"extras"},{"doc":"IteratingIteraptor.each(term, fun/1, opts) — iterates the nested structure, yielding the key and value. The returned from the function value is discarded.function argument: {key, value} tupleoptions: yield: [:all, :maps, :lists, :none], :none is the defaultreturn value: selfMapping and ReducingIteraptor.map(term, fun/1, opts) — iterates the nested structure, yielding the key and value. The value, returned from the block should be either a single value or a {key, value} tuple.function argument: {key, value} tupleoptions: yield: [:all, :maps, :lists, :none], :none is the defaultreturn value: mappedIteraptor.reduce(term, fun/2, opts) — iterates the nested structure, yielding the key and value. The value, returned from the block should be an accumulator value.function arguments: {key, value}, acc pairoptions: yield: [:all, :maps, :lists, :none], :none is the defaultreturn value: accumulatorIteraptor.map_reduce(term, fun/2, opts) — iterates the nested structure, yielding the key and value. The value, returned from the block should be a {​{key, value}, acc} value. The first element of this tuple is used for mapping, the last—accumulating the result.function arguments: {key, value}, acc pairoptions: yield: [:all, :maps, :lists, :none], :none is the defaultreturn value: {mapped, accumulator} tupleFilteringIteraptor.filter(term, filter/1, opts) — filters the structure according to the value returned from each iteration (true to leave the element, false to discard.)function argument: {key, value} tupleoptions: yield: [:all, :maps, :lists, :none], :none is the defaultreturn value: filteredFlatteningIteraptor.to_flatmap(term, opts) — flattens the structure into the flatten map/keyword, concatenating keys with a delimiter.options: delimiter: binary(), into: term(), defaults: delimiter: &quot;.&quot;, into: %{}return value: flattenedIteraptor.from_flatmap(term, fun/1, opts) — de-flattens the structure from the flattened map/keyword, splitting keys by a delimiter. An optional transformer function might be called after the value is deflattened.function argument: {key, value} tupleoptions: delimiter: binary(), into: term(), defaults: delimiter: &quot;.&quot;, into: %{}return value: Map.t | Keyword.t | List.t","ref":"intro.html#in-details","title":"Iteraptor - In Details","type":"extras"}]